import React from "react";
import ReactDOM from "react-dom";
import { defaultProps } from "./props/defaultProps";
import { propTypes } from "./props/propTypes";
import debounce from "./utils/debounce";
import calculateNodeHeight from "./utils/calculateNodeHeight";
import ownerWindow from "./utils/ownerWindow";
import updateState from "./utils/updateState";
/* eslint css-modules/no-unused-class: [2, { markAsUsed: ['resizeX', 'resizeY', 'noresize', 'resizeboth', 'xsmall', 'small', 'medium', 'large', 'xlarge', 'default', 'primary'] }] */

import style from "./Textarea.module.css";
export default class Textarea extends React.Component {
  constructor(props) {
    super(props);
    this.hiddenTextareaRef = /*#__PURE__*/React.createRef();
    this.rendersRef = /*#__PURE__*/React.createRef();
    this.sizingData = /*#__PURE__*/React.createRef(null);
    this.rAF = null;
    this.resizeObserver = null;
    this.debounceHandleResize = null;
    this.onBlur = this.onBlur.bind(this);
    this.onKeyDown = this.onKeyDown.bind(this);
    this.inputRef = this.inputRef.bind(this);
    this.isEmpty = this.isEmpty.bind(this);
    this.syncHeight = this.syncHeight.bind(this);
    this.onChange = this.onChange.bind(this);
    this.handleResize = this.handleResize.bind(this);
    this.rAFHandleResize = this.rAFHandleResize.bind(this);
    this.mountingPhase = this.mountingPhase.bind(this);
    this.unMountingPhase = this.unMountingPhase.bind(this);
  }

  onBlur(e) {
    e && e.preventDefault();
    let {
      onBlur
    } = this.props;
    onBlur && onBlur(e.target.value, e);
  }

  onKeyDown(e) {
    let {
      onKeyDown
    } = this.props;
    onKeyDown && onKeyDown(e);
  }

  inputRef(e) {
    let {
      getRef
    } = this.props;
    this.textareaRef = e;
    getRef && getRef(e);
  }

  isEmpty(obj) {
    return obj === undefined || obj === null || Object.keys(obj).length === 0 || obj.height === 0 && !obj.overflow;
  }

  syncHeight() {
    const {
      minRows,
      maxRows
    } = this.props;
    const {
      height,
      overflow
    } = calculateNodeHeight({
      minRows: minRows,
      maxRows: maxRows,
      textareaEle: this.textareaRef,
      hiddenTextareaEle: this.hiddenTextareaRef.current
    });
    const newState = {
      height: height,
      overflow: overflow
    };

    if (this.isEmpty(newState)) {
      return;
    }

    this.sizingData.current = updateState(this.sizingData.current, newState, this.rendersRef); // this.textareaRef.style.setProperty('height',`${this.sizingData.current.height}px`);

    this.textareaRef.style.height = `${this.sizingData.current.height}px`;
    this.textareaRef.style.overflowY = `${this.sizingData.current.overflow ? 'hidden' : 'auto'}`;
  }

  onChange(e) {
    e && e.preventDefault();
    let {
      onChange
    } = this.props;
    this.rendersRef.current = 0;
    const value = e.target.value;
    requestAnimationFrame(() => {
      this.syncHeight();
      onChange && onChange(value, e);
    });
  }

  handleResize() {
    this.rendersRef.current = 0;
    this.syncHeight();
  }

  rAFHandleResize() {
    cancelAnimationFrame(this.rAF);
    this.rAF = requestAnimationFrame(() => {
      this.handleResize();
    });
  }

  mountingPhase() {
    const input = this.textareaRef;
    const containerWindow = ownerWindow(input);
    this.debounceHandleResize = debounce(this.handleResize);
    containerWindow.addEventListener("resize", this.debounceHandleResize);

    if (typeof ResizeObserver !== "undefined") {
      this.resizeObserver = new ResizeObserver(this.rAFHandleResize);
      this.resizeObserver.observe(input);
    }
  }

  unMountingPhase() {
    const input = this.textareaRef;
    const containerWindow = ownerWindow(input);
    this.debounceHandleResize.clear();
    cancelAnimationFrame(this.rAF);
    containerWindow.removeEventListener("resize", this.debounceHandleResize);

    if (this.resizeObserver) {
      this.resizeObserver.disconnect();
    }
  }

  componentDidMount() {
    this.sizingData.current = {
      height: 0,
      overflow: false
    };
    this.mountingPhase();
  }

  componentWillUnmount() {
    this.sizingData.current = null;
    this.unMountingPhase();
  }

  componentDidUpdate(prevProps) {
    if (prevProps.text !== this.props.text) {
      this.rendersRef.current = 0;
      this.syncHeight();
    }
  }

  render() {
    let {
      size,
      placeHolder,
      needBorder,
      text,
      isDisabled,
      resize,
      maxLength,
      animated,
      variant,
      getRef,
      onFocus,
      dataId,
      dataSelectorId,
      isReadOnly,
      needAppearance,
      needReadOnlyStyle,
      borderColor,
      needEffect,
      autoFocus,
      htmlId,
      a11y,
      customClass,
      minRows
    } = this.props;
    let {
      ariaLabel,
      ariaLabelledby
    } = a11y;
    let resizes = {
      horizontal: "resizeX",
      vertical: "resizeY",
      both: "resizeboth",
      none: "noresize"
    };
    let options = {};

    if (isReadOnly) {
      options.readOnly = "readOnly";
    }

    if (isDisabled) {
      options.disabled = "disabled";
    }

    if (autoFocus) {
      options.autoFocus = true;
    }

    let classList = needAppearance ? `${style.container} ${style[size]} ${style[variant]} ${needBorder ? style.needBorder : style.noBorder} ${resize ? style[resizes[resize]] : style[resizes.none]}  ${animated ? `${style[`${size}animated`]}` : ""}   ${isDisabled && !needEffect || isReadOnly && !needEffect ? "" : style.effect}` : `${style.basic}`;
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("textarea", {
      "aria-label": ariaLabel,
      "aria-labelledby": ariaLabelledby,
      className: `${customClass} ${isReadOnly && needReadOnlyStyle ? style.readonly : ""} ${classList} ${style[`borderColor_${borderColor}`]}`,
      placeholder: placeHolder,
      ...options,
      "data-id": dataId,
      "data-test-id": dataId,
      maxLength: maxLength,
      onChange: this.onChange,
      onKeyDown: this.onKeyDown,
      onFocus: onFocus,
      onBlur: this.onBlur,
      ref: this.inputRef,
      value: text,
      id: htmlId,
      "data-selector-id": dataSelectorId,
      rows: minRows
    }), /*#__PURE__*/React.createElement("textarea", {
      "aria-hidden": true,
      readOnly: true,
      ref: this.hiddenTextareaRef,
      tabIndex: -1,
      className: `${customClass} ${isReadOnly && needReadOnlyStyle ? style.readonly : ""} ${classList} ${style[`borderColor_${borderColor}`]} ${style.forceHiddenStyle}`
    }));
  }

}
Textarea.defaultProps = defaultProps;
Textarea.propTypes = propTypes; // if (__DOCS__) {
//   Textarea.docs = {
//     componentGroup: 'Form Elements',
//     folderName: 'Style Guide',
//     external: true,
//     description: ' '
//   };
// }