import getSizingData from "./getSizingData";
export default function calculateNodeHeight(_ref) {
  let {
    minRows = 1,
    maxRows = 5,
    textareaEle,
    hiddenTextareaEle
  } = _ref;
  const {
    computedStyle,
    paddingSize,
    borderSize,
    boxSizing
  } = getSizingData(textareaEle);
  const isBorderBox = boxSizing === "border-box"; // If input's width is shrunk and it's not visible, don't sync height.

  if (computedStyle.width === '0px') {
    return {
      height: 0
    };
  }

  let hiddenTextarea = hiddenTextareaEle;
  hiddenTextarea.style.width = computedStyle.width;
  hiddenTextarea.value = textareaEle.value || textareaEle.placeholder || "x";

  if (hiddenTextarea.value.slice(-1) === "\n") {
    //   // Certain fonts which overflow the line height will cause the textarea
    //   // to report a different scrollHeight depending on whether the last line
    //   // is empty. Make it non-empty to avoid this issue.
    hiddenTextarea.value += " ";
  } // The height of the inner content


  const innerHeight = hiddenTextarea.scrollHeight; // Measure height of a textarea with a single row

  hiddenTextarea.value = "x";
  const singleRowHeight = hiddenTextarea.scrollHeight - paddingSize || parseInt(computedStyle.lineHeight); // The height of the outer content

  let outerHeight = innerHeight;

  if (minRows) {
    outerHeight = Math.max(Number(minRows) * singleRowHeight, outerHeight);
  }

  if (maxRows) {
    outerHeight = Math.min(Number(maxRows) * singleRowHeight, outerHeight);
  }

  outerHeight = Math.max(outerHeight, singleRowHeight); // Take the box sizing into account for applying this value as a style.

  const height = outerHeight + (isBorderBox ? paddingSize + borderSize : 0);
  const overflow = Math.abs(outerHeight - innerHeight) <= 1;
  return {
    height,
    overflow
  };
}