"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = calculateNodeHeight;

var _getSizingData2 = _interopRequireDefault(require("./getSizingData"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function calculateNodeHeight(_ref) {
  var _ref$minRows = _ref.minRows,
      minRows = _ref$minRows === void 0 ? 1 : _ref$minRows,
      _ref$maxRows = _ref.maxRows,
      maxRows = _ref$maxRows === void 0 ? 5 : _ref$maxRows,
      textareaEle = _ref.textareaEle,
      hiddenTextareaEle = _ref.hiddenTextareaEle;

  var _getSizingData = (0, _getSizingData2["default"])(textareaEle),
      computedStyle = _getSizingData.computedStyle,
      paddingSize = _getSizingData.paddingSize,
      borderSize = _getSizingData.borderSize,
      boxSizing = _getSizingData.boxSizing;

  var isBorderBox = boxSizing === "border-box"; // If input's width is shrunk and it's not visible, don't sync height.

  if (computedStyle.width === '0px') {
    return {
      height: 0
    };
  }

  var hiddenTextarea = hiddenTextareaEle;
  hiddenTextarea.style.width = computedStyle.width;
  hiddenTextarea.value = textareaEle.value || textareaEle.placeholder || "x";

  if (hiddenTextarea.value.slice(-1) === "\n") {
    //   // Certain fonts which overflow the line height will cause the textarea
    //   // to report a different scrollHeight depending on whether the last line
    //   // is empty. Make it non-empty to avoid this issue.
    hiddenTextarea.value += " ";
  } // The height of the inner content


  var innerHeight = hiddenTextarea.scrollHeight; // Measure height of a textarea with a single row

  hiddenTextarea.value = "x";
  var singleRowHeight = hiddenTextarea.scrollHeight - paddingSize || parseInt(computedStyle.lineHeight); // The height of the outer content

  var outerHeight = innerHeight;

  if (minRows) {
    outerHeight = Math.max(Number(minRows) * singleRowHeight, outerHeight);
  }

  if (maxRows) {
    outerHeight = Math.min(Number(maxRows) * singleRowHeight, outerHeight);
  }

  outerHeight = Math.max(outerHeight, singleRowHeight); // Take the box sizing into account for applying this value as a style.

  var height = outerHeight + (isBorderBox ? paddingSize + borderSize : 0);
  var overflow = Math.abs(outerHeight - innerHeight) <= 1;
  return {
    height: height,
    overflow: overflow
  };
}